/* 8 Queens - find one solution to the 8 queens problem and print it.
   Change SIZE to any other positive integer for the n-queen problem */

#include <stdio.h>

#define SIZE 8
#define EMPTY 0
#define QUEEN 1

#define TRUE 1
#define FALSE 0

void print_board(int board[][SIZE]);
// check - receive a board and a location. Returns TRUE if 
// this location is "safe" for a queeen, FaALSE otherwise
int check(int board[][SIZE], int row, int col);
// check_diag - helper function, used in check
int check_diag(int board[][SIZE], int row, int col, int add_row, int add_col);
// solve - recursive solution
int solve(int board[][SIZE], int row);

int main()
{
	int board[SIZE][SIZE] = {EMPTY};
	
	if (solve(board, 0))
		print_board(board);
	else
		printf("Failed to solve the %d queens problem\n");

	return 0;
}

int solve(int board[][SIZE], int row)
{
	int col;

	if (row == SIZE)
		return TRUE;

	for (col = 0; col < SIZE; ++col)
	{
		if (check(board, row, col))
		{
			board[row][col] = QUEEN;
			if (solve(board, row + 1))
				return TRUE;
			// else - reset last assignment and try next possible column
			board[row][col] = EMPTY;
		}
	}
	return FALSE;
}

void print_board(int board[][SIZE])
{
	int row, col;
	
	for (row = 0; row < SIZE; ++row)
	{
		for (col = 0; col < SIZE; ++col)
		{
			if (board[row][col] == EMPTY)
				printf("%c ", '_');
			else
				printf("%c ", 'Q');
		}
		printf("\n");
	}
}

int check(int board[][SIZE], int row, int col)
{
	return (check_helper(board, row, col,  1, 0) && check_helper(board, row, col, -1,  0)  && // check column
			check_helper(board, row, col,  0, 1) && check_helper(board, row, col,  0, -1)  && // check row
			check_helper(board, row, col,  1, 1) && check_helper(board, row, col,  1, -1)  && // check diagonal
			check_helper(board, row, col, -1, 1) &&	check_helper(board, row, col, -1, -1));   // check other diagonal
}

// check_helper - returns FALSE if a QUEEN is found on the path
int check_helper(int board[][SIZE], int row, int col, int add_row, int add_col)
{
	int i, j;

	i = row + add_row;
	j = col + add_col;
	while ((i >= 0) && (i < SIZE) && (j >= 0) && (j < SIZE))
	{
		if (board[i][j] == QUEEN)
			return FALSE;
		i += add_row;
		j += add_col;
	}
	return TRUE;
}